package com.umall.shop.home.service;

import com.netflix.hystrix.contrib.javanica.annotation.HystrixCommand;
import com.umall.shop.home.domain.Carousel;
import com.umall.shop.home.domain.Category;
import com.umall.shop.home.domain.CustomService;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.List;

/**
 * @author yangck
 * @create 2017-02-15 17:40
 */
@Service
public class HomeService {

    private final Logger logger = LogManager.getLogger(getClass());

    @Autowired
    private RestTemplate restTemplate;

    @HystrixCommand(fallbackMethod = "getCategoryFallback")
    public Category getCategory(Long id) {
//        System.out.println("getCategory");
        return restTemplate.getForEntity("http://CATEGORY-SERVICE/category/" + id, Category.class).getBody();
    }

    private Category getCategoryFallback(Long id) {
        // throw new ServiceUnavailableException("CATEGORY-SERVICE");
        logger.error("CATEGORY-SERVICE unavailable");
        return new Category();
    }

    @HystrixCommand(fallbackMethod = "getCarousels")
    public List<Carousel> getCarousels(String usedFor) {
        return restTemplate.getForEntity("http://CAROUSEL-SERVICE/carousels?usedFor=" + usedFor, List.class).getBody();
    }

    private List<Carousel> getCarousels() {
        logger.error("CAROUSEL-SERVICE unavailable");
        return new ArrayList<>();
    }

    /**
     * 获取商城客服信息
     *
     * @param enuu  商城企业UU信息
     */
    @HystrixCommand(fallbackMethod = "getCustomService")
    public CustomService getCustomService(Long enuu) {
        return restTemplate.getForEntity(String.format("http://MALL-CUSTOM-SERVICE/customService/cus/%d", enuu), CustomService.class).getBody();
    }

    private CustomService getCustomService() {
        logger.error("MALL-CUSTOM-SERVICE unavailable");
        CustomService customService = new CustomService();
        customService.setTelephone("0755-86564892");
        customService.setAddress("深圳市南山区英唐大厦6楼");
        return customService;
    }

    /**
     * 根据用户的企业UU和用户UU获取购物车中商品的数量
     *
     * @param enuu      用户企业UU
     * @param userUU    用户UU
     * @return      购物车中商品的数量，非负
     */
    @HystrixCommand(fallbackMethod = "getCartCount")
    public Integer getCartCount(Long enuu, Long userUU) {
        Integer cartCount = restTemplate.getForEntity(String.format("http://MALL-CART-SERVICE/cart/count?enuu=%d&useruu=%d", enuu, userUU), Integer.class).getBody();
        return cartCount == null ? 0 : cartCount;
    }

    private Integer getCartCount() {
        logger.error("MALL-CART-SERVICE unavailable");
        return -1;
    }
}
